==> qps-1.10.1a8.tar.bz2: qps-1.10.1a8/htable.h <==
// htable.h				emacs, this is written in -*-c++-*-
//
// This program is free software. See the file COPYING for details.
// Author: Mattias Engdegrd, 1997-1999

// This file defines the HeadedTable class.
// A HeadedTable is a QTableView with one line of headers over each column.

#ifndef HTABLE_H
#define HTABLE_H

#include "qttableview.h"
#include <qscrollbar.h>
#include <qtooltip.h>
#include <QHash>
#include <qbitarray.h>
#include <qpixmap.h>
#include "svec.h"

class HeadedTable;
class CellAttribute;


class TableHead : public QtTableView
{
	Q_OBJECT
	public:
		TableHead(HeadedTable *parent = 0);
		void tip(const QPoint &pos);
        virtual QString text(int row,int col);

	protected slots:
		void scrollSideways(int);

	protected:
		virtual void paintCell(QPainter *p, int row, int col,bool update);
		virtual int cellWidth(int col);
		virtual void mousePressEvent(QMouseEvent *e);
		virtual void mouseReleaseEvent(QMouseEvent *e);
		virtual void mouseMoveEvent(QMouseEvent *e);
    	virtual void eraseRight(QPainter *,QRect &r);

		void paintHeading(int x, int col);

		static const int drag_threshold = 10; // movement before title drag starts
		
		HeadedTable *htable;	// to access parent class
		QPoint press;
		bool left_pressed;
		int click_col;		// physical column clicked in
		bool dragging;
		int drag_pos;		// previous dragging position
		int drag_offset;
signals:
		void rightClicked(QPoint where, int col);
		void toolTip(QPoint where, int col);

		friend class HeadedTable;
};

class TableBody : public QtTableView
{
	Q_OBJECT
	public:
		TableBody(HeadedTable *parent = 0);
		void drawGhostCol(int x, int w);
		// int zerowidth;		// width of the digit 0
		virtual QString text(int row, int col);

	protected slots:
		// for keyboard accelerators
		void scrollUp();
		void scrollDown();
		void scrollLeft();
		void scrollRight();
		void pageUp();
		void pageDown();
		void jumpTop();
		void jumpBottom();
		void centerVertically(int row);
		void showRange(int from, int to);

	protected:
		virtual void paintCell(QPainter *p, int row, int col,bool forced);
		virtual int cellWidth(int col);
		void mousePressEvent(QMouseEvent *e);
		void mouseDoubleClickEvent(QMouseEvent *e);
		void mouseMoveEvent(QMouseEvent *e);
		void mouseReleaseEvent(QMouseEvent *);
		void wheelEvent ( QWheelEvent * e );

		void timerEvent(QTimerEvent *);
		void updateRow(int row);
		void dragSelectTo(int row);
		QColor getXorColor();
		HeadedTable *htable;// to access parent class
		int first_drag_row;		// row where drag started
		int prev_drag_row;		// row where drag was at last event
		bool autoscrolling;		// true if we are autoscrolling right now
		enum { UP, DOWN } scrolldir;
		static const int scroll_delay = 10; // time delay when autoscrolling, in ms
		bool gadget_click;		// whether mouse press was on folding gadget
		QColor background;
	private:
		QColor color_text;
	friend class HeadedTable;
};

// table options (bits to be ORed together)
#define HTBL_ROW_SELECTION 	1	// rows are selectable
#define HTBL_ROW_DOUBLE_CLICK 	2	// rows can be double-clicked
#define HTBL_ROW_CONTEXT_MENU 	4	// right button menu on rows
#define HTBL_HEADING_TOOLTIPS 	8	// tooltips on headings
#define HTBL_HEADING_CONTEXT_MENU 16 	// right button menu on headings
#define HTBL_HEADING_CLICK 	32	// able to click on heading (change sort order)
#define HTBL_REORDER_COLS 	64	// able to reorder columns

class HeadedTable : public QWidget
{
	Q_OBJECT
	public:
		HeadedTable(QWidget *parent, int opts=0);
		~HeadedTable();

		TableHead *head;
		TableBody *body;

		enum NodeState { Leaf, Open, Closed };

		void repaint_changed();
		void repaintAll();
		
        void updateTableSize();
		void updateColWidths();	
        int  updateColWidth(int col);

		void resetWidths();
		void resetWidth(int col) { max_widths[col] = -1; }; // dont use ?

		void setSortedCol(int col);
		int sortedCol() { return sorted_col; }
		int numRows() { return nrows; }
		int numCols() { return ncols; }
		void setNumRows(int rows);
		void setNumCols(int cols);
		int  clickedColumn(){return head->click_col;}
		void deleteCol(int col, bool update = TRUE);
		void topAndRepaint();
		int leftCell() { return body->leftCell(); };
		int lastColVisible() { return body->lastColVisible(); };
		int topCell() { return body->topCell(); };
		int lastRowVisible() { return body->lastRowVisible(); };
		void updateCell(int row, int col, bool erase=false);
		void updateHeading(int col) { head->updateCell(0, col); };
		int bodyTextWidth(const char *s);
		int headTextWidth(const char *s);
		void setAutoUpdate(bool update)	{ head->setAutoUpdate(update); body->setAutoUpdate(update); };
		void centerVertically(int row) { body->centerVertically(row); };
		void showRange(int from, int to) { body->showRange(from, to); };
		void selectionNotify();
		bool locked(){ return head->dragging; };
		void repaintColumns(int col0, int col1 = -1);
		void setTreeMode(bool tm);
		bool treeMode() { return treemode; };
		void enableFolding(bool enable);
		bool foldingEnabled() { return folding; };
		void enableLines(bool enable);
		bool linesEnabled() { return lines; }
		int	tableWidth() const { return body->totalWidth()+ body->verticalScrollBar()->width(); }
        bool isCellContentsChanged(int row,int col,bool head);
		

		void selectOnly(int row);
		int numSelected() { return 0; }
		void clearAllSelections();
		virtual void setSelected(int row, bool sel){};
		virtual bool isSelected(int row) { return false; }
	signals:
		void titleClicked(int col);
		void selectionChanged(const Svec<int> *rows);
		void doubleClicked(int row);
		void rightClicked(QPoint where);
		void foldSubTree(int row);
		void colMoved(int col, int place);
		

	protected slots:
		void selectAll();
	protected:
		void fontChange( const QFont & oldFont ) ;
		// These must be implemented in subclasses
		virtual QString title(int col) = 0;
		virtual QString text(int row, int col) = 0;
		virtual char *total_selectedRow(int col);
		virtual int sizeHintForColumn(int col) const;

		// colWidth returns width in digit units; negative means variable width.
		virtual int colWidth(int col) = 0;	//head_width
		virtual int alignment(int col);

		// may be reimplemented (default exists)
		virtual void moveCol(int col, int place);

		virtual QString tipText(int col);
		virtual void drawCellContents(int row, int col, int w, int h,  QPainter *painter);
		virtual int rowDepth(int row){ return 0;} ;
		virtual NodeState folded(int row){ return Leaf;};
		virtual int parentRow(int row){return 0;};
		virtual bool lastChild(int row){return false;};

		void resizeEvent(QResizeEvent *);

		bool treemode;
		int treestep;		// indentation for each tree level

	private:
		// to make signals originate from the right object
		void emit_click_signal(int col);
		void emit_double_click_signal(int row);
		void emit_right_click_signal(QPoint where);
		void emit_fold(int row);

		inline int computedWidth(int col);
		int colOffset(int col);
		inline int colXPos(int col);
		void updateCols(int deltacols, int place, bool update);


		int 	alignment_col[64];
		int 	headwidth_col[64];
		int 	max_widths[64];		// row widths, indexed by columns
		//Svec<int> widths;		// row widths, indexed by columns

		int sorted_col;		    // column to emphasize
		int reversed_sort;		// true if sorting backwards
		int options;
		int nrows;
		int nrows_prev;
		int ncols_prev;
		int ncols;	

		QHash<int,CellAttribute *> cached_attr; // indexed by (row << 16) + log_col
		QHash<int,CellAttribute * > cached_attr_h; // indexed by (row << 16) + log_col

		// text cache is in logical columns (reorder causes no flush)
		bool folding;		// true if folding gadgets are used
		bool lines;			// true if tree lines are drawn
		int gadget_space;		// horizontal space for show/hide tree gadget
	
	friend class TableHead;
	friend class TableBody;
};
#endif	// HTABLE_H

==> qps-1.10.3.tar.bz2: qps-1.10.3/htable.h <==
// htable.h				emacs, this is written in -*-c++-*-
//
// This program is free software. See the file COPYING for details.
// Author: Mattias Engdegrd, 1997-1999

// This file defines the HeadedTable class.
// A HeadedTable is a QTableView with one line of headers over each column.

#ifndef HTABLE_H
#define HTABLE_H

#include "qttableview.h"
#include <qscrollbar.h>
#include <qtooltip.h>
#include <QHash>
#include <qbitarray.h>
#include <qpixmap.h>
#include "svec.h"

// table options (bits to be ORed together)
#define HTBL_ROW_SELECTION		1	// rows are selectable
#define HTBL_ROW_DOUBLE_CLICK 	2	// rows can be double-clicked
#define HTBL_ROW_CONTEXT_MENU 	4	// right button menu on rows
#define HTBL_HEADING_TOOLTIPS 	8	// tooltips on headings
#define HTBL_HEADING_CONTEXT_MENU 16 	// right button menu on headings
#define HTBL_HEADING_CLICK		32	// able to click on heading (change sort order)
#define HTBL_REORDER_COLS		64	// able to reorder columns


class HeadedTable;
class CellAttribute;


class FloatingHead : public QWidget
{
	Q_OBJECT
	public:
		FloatingHead(QWidget *parent);
		QPixmap *pix;
		void setTitle(QString str,int w,int h);
		QString title;
	protected slots:
	protected :
		virtual void paintEvent(  QPaintEvent * event ); 
};

class TableHead : public QtTableView
{
	Q_OBJECT
	public:
		TableHead(HeadedTable *parent = 0);
        virtual QString text(int row,int col);

	protected slots:
		void scrollSideways(int);

	protected:
		virtual void paintCell(QPainter *p, int row, int col,bool update);
		virtual int	cellWidth(int col);
		virtual void mousePressEvent(QMouseEvent *e);
		virtual void mouseReleaseEvent(QMouseEvent *e);
		virtual void mouseMoveEvent(QMouseEvent *e);
    	virtual void eraseRight(QPainter *,QRect &r);

		FloatingHead *floatHead;

		HeadedTable *htable;	// to access parent class
		QPoint press;
		int click_col;		// physical column clicked in
		bool dragging;
		int drag_pos;		// previous dragging position
		int drag_offset;
	signals:
		void rightClicked(QPoint where, int col);
		void toolTip(QPoint where, int col);
		
		friend class HeadedTable;
};

class TableBody : public QtTableView
{
	Q_OBJECT
	public:
		TableBody(HeadedTable *parent = 0);
		void drawGhostCol(int x, int w);
		// int zerowidth;		// width of the digit 0
		virtual QString text(int row, int col);

	protected slots:
		// for keyboard accelerators
		void scrollUp();
		void scrollDown();
		void scrollLeft();
		void scrollRight();
		void pageUp();
		void pageDown();
		void jumpTop();
		void jumpBottom();
		void centerVertically(int row);
		void showRange(int from, int to);

	protected:
		virtual void paintCell(QPainter *p, int row, int col,bool forced);
		virtual int cellWidth(int col);
		void mousePressEvent(QMouseEvent *e);
		void mouseDoubleClickEvent(QMouseEvent *e);
		void mouseMoveEvent(QMouseEvent *e);
		void mouseReleaseEvent(QMouseEvent *);
		void wheelEvent ( QWheelEvent * e );

		void timerEvent(QTimerEvent *);
		void updateRow(int row);
		void dragSelectTo(int row);
		QColor getXorColor();
		HeadedTable *htable;// to access parent class
		int first_drag_row;		// row where drag started
		int prev_drag_row;		// row where drag was at last event
		bool autoscrolling;		// true if we are autoscrolling right now
		enum { UP, DOWN } scrolldir;
		static const int scroll_delay = 10; // time delay when autoscrolling, in ms
		bool gadget_click;		// whether mouse press was on folding gadget
		QColor background;
	private:
		QColor color_text;
	friend class HeadedTable;
};

class HeadedTable : public QWidget
{
	Q_OBJECT
	public:
		HeadedTable(QWidget *parent, int opts=0);
		~HeadedTable();

		TableHead *head;
		TableBody *body;

		CellAttribute *attr; //temporarily buffer
		enum NodeState { Leaf, Open, Closed };

		void repaint_changed();
        
		void updateTableSize();
		void updateColWidths();	
        int  updateColWidth(int col);

		void resetWidths();
		void resetWidth(int col) { max_widths[col] = -1; }; // dont use ?

		void setSortedCol(int col);
		int sortedCol() { return sorted_col; }
		int numRows() { return nrows; }
		int numCols() { return ncols; }
		void setNumRows(int rows);
		void setNumCols(int cols);
		int  clickedColumn(){return head->click_col;}
		void deleteCol(int col, bool update = TRUE);
		void topAndRepaint();
		int leftCell() { return body->leftCell(); };
		int lastColVisible() { return body->lastColVisible(); };
		int topCell() { return body->topCell(); };
		int lastRowVisible() { return body->lastRowVisible(); };
		void updateCell(int row, int col, bool erase=false);
		void updateHeading(int col) { /* head->updateCell(0, col); */ };
		int bodyTextWidth(const char *s);
		int headTextWidth(const char *s);
		void setAutoUpdate(bool update)	{ head->setAutoUpdate(update); body->setAutoUpdate(update); };
		void centerVertically(int row) { body->centerVertically(row); };
		void showRange(int from, int to) { body->showRange(from, to); };
		void selectionNotify();
		void repaintColumns(int col0, int col1 = -1);
		void setTreeMode(bool tm);
		bool treeMode() { return treemode; };
		void enableFolding(bool enable);
		bool foldingEnabled() { return folding; };
		void enableLines(bool enable);
//DEL	bool linesEnabled() { return lines; }
		int	tableWidth() const { return body->totalWidth()+ body->verticalScrollBar()->width(); }
        bool isCellContentsChanged(int row,int col,bool head);
		

		void selectOnly(int row);
		int numSelected() { return 0; }
		void clearAllSelections();

		virtual void setSelected(int row, bool sel){};
		virtual bool isSelected(int row) { return false; }
	signals:
		void titleClicked(int col);
		void selectionChanged(const Svec<int> *rows);
		void doubleClicked(int row);
		void rightClicked(QPoint where);
		void foldSubTree(int row);
		void colMoved(int col, int place);
		

	public slots:
		void selectAll();
		void repaintAll();
	protected:
		void fontChange( const QFont & oldFont ) ;
		// These must be implemented in subclasses
		virtual QString title(int col) = 0;
		virtual QString text(int row, int col) = 0;
		virtual char *total_selectedRow(int col);
		virtual int sizeHintForColumn(int col) const;
    	virtual void	paintEvent( QPaintEvent * );

		// colWidth returns width in digit units; negative means variable width.
		virtual int colWidth(int col) = 0;	//head_width
		virtual int alignment(int col);

		// may be reimplemented (default exists)
		virtual void moveCol(int col, int place);

		virtual QString tipText(int col);
		virtual void drawCellContents(int row, int col, int w, int h,  QPainter *painter);
		virtual int rowDepth(int row){ return 0;} ;
		virtual NodeState folded(int row){ return Leaf;};
		virtual int parentRow(int row){return 0;};
		virtual bool lastChild(int row){return false;};
		virtual bool columnMovable(int col){return true;};
		//virtual bool modified(int row){return true;};

		void resizeEvent(QResizeEvent *);

		bool treemode;
		int treestep;		// indentation for each tree level

	private:
		// to make signals originate from the right object
		void emit_click_signal(int col);
		void emit_double_click_signal(int row);
		void emit_right_click_signal(QPoint where);
		void emit_fold(int row);

		inline int computedWidth(int col);
		int colOffset(int col);
		inline int colXPos(int col);
		void updateCols(int deltacols, int place, bool update);


		int 	alignment_col[64];
		int 	headwidth_col[64];
		int 	max_widths[64];		// row widths, indexed by columns
		//Svec<int> widths;		// row widths, indexed by columns

		int sorted_col;		    // column to emphasize
		int reversed_sort;		// true if sorting backwards
		int options;
		int nrows;
		int nrows_prev;
		int ncols_prev;
		int ncols;	

		QHash<int,CellAttribute *>	cached_attr; // indexed by (row << 16) + log_col
		QHash<int,CellAttribute * > cached_attr_h; // indexed by (row << 16) + log_col

		// text cache is in logical columns (reorder causes no flush)
		bool folding;		// true if folding gadgets are used
		bool lines;			// true if tree lines are drawn
		int gadget_space;		// horizontal space for show/hide tree gadget
	
	friend class TableHead;
	friend class TableBody;
};
#endif	// HTABLE_H
